@@ -22,7 +22,7 @@ import posix
 import time
 from abc import ABCMeta
 from fcntl import ioctl
-from .asm_generic_ioctl import _IOW
+from .linux_spi_spidev import spi_ioc_transfer, SPI_IOC_MESSAGE
 
 
 MAX_BOARDS = 4
@@ -67,8 +67,6 @@ ODR_OFF = 0x00
 INTPOL_HIGH = 0x02  # interupt polarity
 INTPOL_LOW = 0x00
 
-SPI_IOC_MAGIC = 107
-
 SPIDEV = '/dev/spidev'
 SPI_HELP_LINK = \
     "https://github.com/piface/pifacecommon#1-enable-the-spi-module"
@@ -89,7 +87,7 @@ class InputDeviceError(Exception):
 
 
 class DigitalPort(object):
-    """A port on a PiFace Digital product"""
+    """A digital port on a PiFace product."""
     __metaclass__ = ABCMeta
 
     def __init__(self, port, board_num=0):
@@ -103,16 +101,39 @@ class DigitalPort(object):
 
     @property
     def handler(self):
+        """The module that handles this port (can be useful for
+        emulator/testing).
+        """
         return sys.modules[__name__]
 
-
-class DigitalInputPort(DigitalPort):
-    """An input port on a PiFace Digital product"""
-    def __init__(self, port, board_num=0):
-        super().__init__(port, board_num)
-
     @property
     def value(self):
+        """The value of the digital port."""
+        return self.handler.read(self.port, self.board_num)
+
+    @value.setter
+    def value(self, data):
+        return self.handler.write(data, self.port, self.board_num)
+
+
+class DigitalInputPort(DigitalPort):
+    """An digital input port on a PiFace product.
+
+    .. note:: This hides the fact that inputs are active low.
+
+       >>> input_port = pifacecommon.core.GPIOB
+       >>> hex(pifacecommon.core.read(input_port))
+       '0xFF'
+       >>> hex(pifacecommon.core.DigitalInputPort(input_port).value)
+       '0x00'
+    """
+    def __init__(self, port, board_num=0):
+        super().__init__(port, board_num)
+
+    # redefine value property for input
+    @property
+    def value(self):
+        """The value of the digital input port."""
         return 0xFF ^ self.handler.read(self.port, self.board_num)
 
     @value.setter
@@ -121,30 +142,25 @@ class DigitalInputPort(DigitalPort):
 
 
 class DigitalOutputPort(DigitalPort):
-    """An output port on a PiFace Digital product"""
+    """An digital output port on a PiFace product"""
     def __init__(self, port, board_num=0):
         super().__init__(port, board_num)
 
-    @property
-    def value(self):
-        return self.handler.read(self.port, self.board_num)
-
-    @value.setter
-    def value(self, data):
-        return self.handler.write(data, self.port, self.board_num)
-
     def all_on(self):
+        """Turns all outputs on."""
         self.value = 0xFF
 
     def all_off(self):
+        """Turns all outputs off."""
         self.value = 0
 
     def toggle(self):
+        """Toggles all outputs."""
         self.value = 0xFF ^ self.value
 
 
 class DigitalItem(DigitalPort):
-    """An item connected to a pin on a PiFace Digital product.
+    """A digital item connected to a pin on a PiFace product.
     Has most of the same properties of a Digital Port.
     """
     __metaclass__ = ABCMeta
@@ -153,31 +169,9 @@ class DigitalItem(DigitalPort):
         super().__init__(port, board_num)
         self.pin_num = pin_num
 
-
-class DigitalInputItem(DigitalItem):
-    """An input connected to a pin a PiFace Digital product"""
-    def __init__(self, pin_num, port, board_num=0):
-        super().__init__(pin_num, port, board_num)
-
-    @property
-    def value(self):
-        return 1 ^ self.handler.read_bit(
-            self.pin_num,
-            self.port,
-            self.board_num)
-
-    @value.setter
-    def value(self, data):
-        raise InputDeviceError("You cannot set an input's values!")
-
-
-class DigitalOutputItem(DigitalItem):
-    """An output connected to a pin a PiFace Digital product"""
-    def __init__(self, pin_num, port, board_num=0):
-        super().__init__(pin_num, port, board_num)
-
     @property
     def value(self):
+        """The value of the digital item."""
         return self.handler.read_bit(
             self.pin_num,
             self.port,
@@ -191,31 +185,63 @@ class DigitalOutputItem(DigitalItem):
             self.port,
             self.board_num)
 
+
+class DigitalInputItem(DigitalItem):
+    """An digital input connected to a pin a PiFace product.
+
+    .. note:: This hides the fact that inputs are active low.
+
+       >>> input_port = pifacecommon.core.GPIOB
+       >>> pifacecommon.core.read_bit(0, input_port)
+       1
+       >>> hex(pifacecommon.core.DigitalInputItem(0, input_port).value)
+       0
+    """
+    def __init__(self, pin_num, port, board_num=0):
+        super().__init__(pin_num, port, board_num)
+
+    # redefine value property for input
+    @property
+    def value(self):
+        """The inverse value of the digital input item (inputs are active low).
+        """
+        return 1 ^ self.handler.read_bit(
+            self.pin_num,
+            self.port,
+            self.board_num)
+
+    @value.setter
+    def value(self, data):
+        raise InputDeviceError("You cannot set an input's values!")
+
+
+class DigitalOutputItem(DigitalItem):
+    """An output connected to a pin a PiFace Digital product."""
+    def __init__(self, pin_num, port, board_num=0):
+        super().__init__(pin_num, port, board_num)
+
     def turn_on(self):
+        """Sets the digital output item's value to 1."""
         self.value = 1
 
     def turn_off(self):
+        """Sets the digital output item's value to 0."""
         self.value = 0
 
     def toggle(self):
+        """Toggles the digital output item's value."""
         self.value = not self.value
 
 
-class _spi_ioc_transfer(ctypes.Structure):
-    """SPI ioc transfer structure (from linux/spi/spidev.h)"""
-    _fields_ = [
-        ("tx_buf", ctypes.c_uint64),
-        ("rx_buf", ctypes.c_uint64),
-        ("len", ctypes.c_uint32),
-        ("speed_hz", ctypes.c_uint32),
-        ("delay_usecs", ctypes.c_uint16),
-        ("bits_per_word", ctypes.c_uint8),
-        ("cs_change", ctypes.c_uint8),
-        ("pad", ctypes.c_uint32)]
-
-
 def init(bus=0, chip_select=0):
-    """Initialises the PiFace Digital board"""
+    """Initialises the SPI device file descriptor.
+
+    :param bus: The SPI device bus number
+    :type bus: int
+    :param chip_select: The SPI device chip_select number
+    :param chip_select: int
+    :raises: InitError
+    """
     spi_device = "%s%d.%d" % (SPIDEV, bus, chip_select)
     global spidev_fd
     try:
@@ -228,7 +254,7 @@ def init(bus=0, chip_select=0):
 
 
 def deinit():
-    """Closes the spidev file descriptor"""
+    """Closes the SPI device file descriptor."""
     global spidev_fd
     if spidev_fd:
         posix.close(spidev_fd)
@@ -236,7 +262,20 @@ def deinit():
 
 
 def get_bit_mask(bit_num):
-    """Translates a pin num to pin bit mask. First pin is pin0."""
+    """Translates a bit num to bit mask.
+
+    :param bit_num: The bit number.
+    :type bit_num: int
+    :returns: int -- the bit mask
+    :raises: RangeError
+
+    >>> pifacecommon.core.get_bit_mask(0)
+    1
+    >>> pifacecommon.core.get_bit_mask(1)
+    2
+    >>> bin(pifacecommon.core.get_bit_mask(3))
+    '0b1000'
+    """
     if bit_num > 7 or bit_num < 0:
         raise RangeError(
             "Specified bit num (%d) out of range (0-7)." % bit_num)
@@ -245,7 +284,24 @@ def get_bit_mask(bit_num):
 
 
 def get_bit_num(bit_pattern):
-    """Returns the lowest pin num from a given bit pattern"""
+    """Returns the lowest bit num from a given bit pattern. Returns None if no
+    bits set.
+
+    :param bit_pattern: The bit pattern.
+    :type bit_pattern: int
+    :returns: int -- the bit number
+    :returns: None -- no bits set
+
+    >>> pifacecommon.core.get_bit_num(0)
+    None
+    >>> pifacecommon.core.get_bit_num(0b1)
+    0
+    >>> pifacecommon.core.get_bit_num(0b11000)
+    3
+    """
+    if bit_pattern == 0:
+        return None
+
     bit_num = 0  # assume bit 0
     while (bit_pattern & 1) == 0:
         bit_pattern = bit_pattern >> 1
@@ -258,14 +314,33 @@ def get_bit_num(bit_pattern):
 
 
 def read_bit(bit_num, address, board_num=0):
-    """Returns the bit specified from the address"""
+    """Returns the bit specified from the address.
+
+    :param bit_num: The bit number to read from.
+    :type bit_num: int
+    :param address: The address to read from.
+    :type address: int
+    :param board_num: The board number to read from.
+    :type board_num: int
+    :returns: int -- the bit value from the address
+    """
     value = read(address, board_num)
     bit_mask = get_bit_mask(bit_num)
     return 1 if value & bit_mask else 0
 
 
 def write_bit(value, bit_num, address, board_num=0):
-    """Writes the value given to the bit specified"""
+    """Writes the value given to the bit in the address specified.
+
+    :param value: The value to write.
+    :type value: int
+    :param bit_num: The bit number to write to.
+    :type bit_num: int
+    :param address: The address to write to.
+    :type address: int
+    :param board_num: The board number to write to.
+    :type board_num: int
+    """
     bit_mask = get_bit_mask(bit_num)
     old_byte = read(address, board_num)
      # generate the new byte
@@ -277,49 +352,79 @@ def write_bit(value, bit_num, address, board_num=0):
 
 
 def __get_device_opcode(board_num, read_write_cmd):
-    """Returns the device opcode (as a byte)"""
+    """Returns the device opcode (as a byte).
+
+    :param board_num: The board number to generate the opcode from.
+    :type board_num: int
+    :param read_write_cmd: Read or write command.
+    :type read_write_cmd: int
+    """
     board_addr_pattern = (board_num << 1) & 0xE  # 1 -> 0b0010, 3 -> 0b0110
     rw_cmd_pattern = read_write_cmd & 1  # make sure it's just 1 bit long
     return 0x40 | board_addr_pattern | rw_cmd_pattern
 
 
 def read(address, board_num=0):
-    """Reads from the address specified"""
+    """Returns the value of the address specified.
+
+    :param address: The address to read from.
+    :type address: int
+    :param board_num: The board number to read from.
+    :type board_num: int
+    """
     devopcode = __get_device_opcode(board_num, READ_CMD)
-    op, addr, data = spisend((devopcode, address, 0))  # data byte is not used
+    op, addr, data = spisend(bytes((devopcode, address, 0)))
     return data
 
 
 def write(data, address, board_num=0):
-    """Writes data to the address specified"""
+    """Writes data to the address specified.
+
+    :param data: The data to write.
+    :type data: int
+    :param address: The address to write to.
+    :type address: int
+    :param board_num: The board number to write to.
+    :type board_num: int
+    """
     devopcode = __get_device_opcode(board_num, WRITE_CMD)
-    op, addr, data = spisend((devopcode, address, data))
+    op, addr, data = spisend(bytes((devopcode, address, data)))
 
 
 def spisend(bytes_to_send):
-    """Sends bytes via the SPI bus"""
+    """Sends bytes via the SPI bus.
+
+    :param bytes_to_send: The bytes to send on the SPI device.
+    :type bytes_to_send: bytes
+    :returns: bytes -- returned bytes from SPI device
+    :raises: InitError
+    """
     global spidev_fd
     if spidev_fd is None:
         raise InitError("Before spisend(), call init().")
 
-     # make some buffer space to store reading/writing
-    write_bytes = bytes(bytes_to_send)
-    wbuffer = ctypes.create_string_buffer(write_bytes, len(write_bytes))
+    # make some buffer space to store reading/writing
+    wbuffer = ctypes.create_string_buffer(bytes_to_send, len(bytes_to_send))
     rbuffer = ctypes.create_string_buffer(len(bytes_to_send))
 
-     # create the spi transfer struct
-    transfer = _spi_ioc_transfer(
+    # create the spi transfer struct
+    transfer = spi_ioc_transfer(
         tx_buf=ctypes.addressof(wbuffer),
         rx_buf=ctypes.addressof(rbuffer),
-        len=ctypes.sizeof(wbuffer))
+        len=ctypes.sizeof(wbuffer)
+    )
 
-     # send the spi command (with a little help from asm-generic
-    iomsg = _IOW(SPI_IOC_MAGIC, 0, ctypes.c_char * ctypes.sizeof(transfer))
-    ioctl(spidev_fd, iomsg, ctypes.addressof(transfer))
+    # send the spi command
+    ioctl(spidev_fd, SPI_IOC_MESSAGE(1), transfer)
     return ctypes.string_at(rbuffer, ctypes.sizeof(rbuffer))
 
 
 def sleep_microseconds(microseconds):
+    """Sleeps for the given number of microseconds.
+
+    :param microseconds: Number of microseconds to sleep for.
+    :type microseconds: int
+    """
     # divide microseconds by 1 million for seconds
     seconds = microseconds / float(1000000)
     time.sleep(seconds)
